function ML_Ustart = init_from_time_simu(lambda,H,sub_harm_factor)
% Initialisation du vecteur ML_Ustart  partir de la simulation temporelle
% lambda = param. de bifurcation 
% H = nbre d'harm.  renvoyer
% sub_harm_factor pour savoir si on veut initialiser sur :
%  - un rgime "normal" : sub_harm_factor = 1
%  - un rgime "doublement de priode" : sub_harm_factor = 2
%  - un rgime "quadruplement de priode" : sub_harm_factor = 4
% 

 addpath('/Volumes/Vergez/Matlab/TOOLBOXES/Chaotic Systems Toolbox/')
 nb_points=20000*sub_harm_factor;
 step_size=1e-2;
 [x,y,z]=rossler(nb_points,0,0.2,0.2,lambda,0,0,0,step_size);
 temps=(0:nb_points-1)*step_size;
 FFT_size = max(2^20,length(x));
 
 FFT_x = fft(x,FFT_size)./nb_points;
 FFT_y = fft(y,FFT_size)./nb_points;
 FFT_z = fft(z,FFT_size)./nb_points;
 
% recherche de la fondamentale
[Y,I] = max(abs(FFT_x(1:length(FFT_x)/2))) ; % recherche de la fondamentale
f=linspace(0,1/step_size*(1-1/length(FFT_x)),length(FFT_x));  % axe frequentiel
freq(1)=f(I);                                % frequence de jeu

FFT_x_h(1)=FFT_x(I);FFT_y_h(1)=FFT_y(I);FFT_z_h(1)=FFT_z(I);

% Recherche des harmoniques
I=I/sub_harm_factor;
for n=1:H,
    [Y,m] = max(abs(FFT_y(round(n*I-10):round(n*I+10))));
    freq(n)=f(m+round(n*I-11));
    FFT_x_h(n)=FFT_x(m+round(n*I-11));
    FFT_y_h(n)=FFT_y(m+round(n*I-11));
    FFT_z_h(n)=FFT_z(m+round(n*I-11));
end

freq
 
% Remplissage du vecteur ML_Ustart;
 ML_Ustart(1:3,1) = [FFT_x(1); FFT_y(1); FFT_z(1)]; % composantes moyennes
 for n=1:H,
    ML_Ustart(3+(n-1)*6+[1:6],1)=[2*real(FFT_x_h(n)); 2*real(FFT_y_h(n)); 2*real(FFT_z_h(n)); -2*imag(FFT_x_h(n)); -2*imag(FFT_y_h(n)); -2*imag(FFT_z_h(n))];
 end
 ML_Ustart(3*(2*H+1)+1,1)=lambda;
  ML_Ustart(3*(2*H+1)+2,1)=2*pi*freq(1);

 
 % Pour vrification graphique
 disp(['La frequence fondamentale estime est :', num2str(freq(1))]);
 figure(100);subplot(2,1,1);plot(temps,[x,y,z]);title('x(t),y(t),z(t)');
 subplot(2,1,2); hold off;
 plot(f,abs(FFT_x)); hold on;plot(f,abs(FFT_y),'y-.');plot(f,abs(FFT_z),'c-.');title('abs(fft(x))');
 plot(freq,abs(FFT_x_h),'Color','r','MarkerSize',6,'LineStyle','o')
 plot(freq,abs(FFT_y_h),'Color','g','MarkerSize',6,'LineStyle','o')
 plot(freq,abs(FFT_z_h),'Color','m','MarkerSize',6,'LineStyle','o')
 
 
end

 